<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Requests;
use App\Models\RoomControlModel;
use App\Models\User;

/*
 * Controller, který přečte z API všechny učebny a jejich počítače.
 * Podle souřadnic X a Y vytvoří obdelníkovou matici každé učebně,
 * která je odeslána do GUI
 */

class GridController extends Controller_api {

    private $maxMatrix = 200; // konstanta maximálního počtu oken v mřížce každé učebny

    /*
     * Odstranění přebytečných prázdných indexů na konci pole s počítači
     */

    private function cutOffset($computers) {
        $offset = $this->maxMatrix;
        while (empty($computers[$offset]['cislo'])) {
            unset($computers[$offset]);
            $offset--;
        }
        return $computers;
    }

    /*
     * První krok, vytvoření prázdného vektoru, clearfix určuje zalomení v grafice - 12(bootstrap layout)
     */

    private function emptyGrid() {
        $pole = array();
        for ($i = 0; $i <= $this->maxMatrix; $i++) {
            $pole[$i]['cislo'] = 0; // default val
            $pole[$i]['clearfix'] = ''; //default val
            if ($i % 12 == 0 && $i !== 0) {
                $pole[$i]['clearfix'] = 'clearfix'; // zalomení v grafice
            }
        }
        return $pole;
    }

    /*
     * Zjištění práv přístupu uživatele do jednotlivých učeben 
     */

    private function checkAllowedClasses(Request $request) {
        if ($request->user()->admin) {
            return $this->api->getAllClassrooms(); // admin stahuje seznam přístupných učeben z api = přístup všude
        } else {
            $classes = array();
            $classes_allowed = RoomControlModel::where('user_id', $request->user()->id)->with('getRoom')->get();
            foreach ($classes_allowed as $class) {
                $classes[] = $class->getRoom->name;
            }
            return $classes; // pole učeben, které má uživatel přístupné
        }
    }

    /*
     * Funkce zajištuje naplnění pomyslné mřížky počítačů v grafice
     */

    public function generateGrid(Request $request) {
        $classes = $this->checkAllowedClasses($request);
        $pc_result = array();

        for ($i = 0; $i < count($classes); $i++) {
            $pc_list = $this->api->getPc($classes[$i]); // počítače v učebně
            $computers = $this->emptyGrid(); // prázdný vektor
            $maxradek = 0;
            $pc_positions = $this->api->getPosition($pc_list); // pole s parametry počítače a souřadnicemi
            if (!empty($pc_positions)) {
                $nickname = substr(reset($pc_list), 0, -2); // přezdívka učebny získaná z názvu počítače(např. rejpal)
                foreach ($pc_positions as $j => $pc_position) {
                    $pcname = $pc_position['nazev'];
                    $pos = (($pc_position['pos_y'] * 12) + $pc_position['pos_x']) - 13; // převedu souřadnice x a y na pozici v 2D vektoru
                    $computers[$pos]['cislo'] = substr($pcname, -2); // číslo počítače
                    $computers[$pos]['class'] = $classes[$i]; // název učebny
                    $computers[$pos]['nazev'] = $pcname; // název počítače
                    if ($pc_position['pos_x'] > $maxradek) { // najdu nejdelší řádek pro určení délky ovládacího panelu v GUI
                        $maxradek = $pc_position['pos_x'];
                    }
                }
                $computers = $this->cutOffset($computers);
                $pc_result[] = ['computers' => $computers, 'maxradek' => $maxradek, 'nazev' => $classes[$i], 'nickname' => $nickname];
                // přidání učebny do pole( s parametry: computers - vektor počítačů, maxradek - int, nazev - string, nickname - string)
            }
        }
        return $pc_result;
    }

    /*
     * Funkci volám z routeru, vracím šablonu
     */

    public function getHomeView(Request $request) {
        $classes = $this->generateGrid($request);
        return view('main')->with('classes', $classes);
    }

    /*
     * Funkci volám z routeru, vracím šablonu
     */

    public function getRulesView(Request $request) {
        if ($this->authorize('RulesEdit', $request->user())) {
            $classes = $this->generateGrid($request);
            return view('rules')->with('classes', $classes);
        } else {
            return redirect()->route('home');
        }
    }

}
